params = {
    "capitalists": (200, 700),
    "cpf_agents": (2, 20),
    "csf_agents": (2, 20),
    "green_energy_owners": (1, 200),
    "brown_energy_owners": (1, 200),
    "unemploymentDole": (100, 2000),
    "tauDesiredConsumption": (0.1, 0.5),
    "ADAPTIVE_PRICE_CHANGE": (0.001, 0.5),
    "ADAPTIVE_PRICE_CHANGE_K": (0.001, 0.5),
    "ADAPTIVE_PRICE_CHANGE_ENERGY": (0.001, 0.5),
    "fossilFuelPriceGrowthRate": (0.001, 0.01),
    "rhoForecast": (0.1, 0.9),
    "rhoForecastEnergy": (0.1, 0.9),
    "rhoForecastCapital": (0.1, 0.9),
    "wageAdjustmentRate": (0.1, 0.9),
    "shareOfWageInConsumption": (0.1, 0.8),
    "shareOfIncomeInConsumption": (0.1, 0.8),
    "shareOfWealthInConsumption": (0.0, 0.8),  # 0.2
    "ownerProportionFromProfits": (0.1, 0.5),
    "repayProportionFromProfits": (0.01, 0.1),
    "wageDividendFromProfits": (0.1, 0.3),
    "capital_multiplier": (1.1, 3),
    "MARKUP_ADJUSTMENT": (0.1, 0.9),
    "rho_labour": (0.1, 0.9),
    "rho_energy": (0.1, 0.9),
    "rho_labour_K": (0.1, 0.9),
    "rho_energy_K": (0.1, 0.9),
}
