#!/usr/bin/env python3
"""
Economic model for CliMaPan-Lab.
This module exports the main EconModel class.
"""

# Import the model from the src directory
from .src.models import EconModel

# Export for convenience
__all__ = ["EconModel"]
